local bit = bit32 or require 'bit'

local function adler32(data, adler)
  adler = adler or 1
  local sum2 = bit.rshift(adler, 16)
  adler = bit.band(adler, 0xFFFF)
  local pos = 0

  while pos < #data do
    local N = math.min(#data-pos, 5552)
    for i = 1+pos, N+pos do
      adler = adler + data:sub(i,i):byte()
      sum2 = sum2 + adler
    end
    pos = pos + N
    adler = adler % 65521
    sum2 = sum2 % 65521
  end

  return bit.bor(bit.lshift(sum2, 16), adler)
end

-- Test
local msg = ("0123456789"):rep(100000)
local t = os.clock()
local a
for i = 1, 1000 do
  a = adler32(msg, a)
end
print(string.format("%X", a))
print(os.clock()-t)
